import 'package:meta/meta.dart';
import 'package:recase/recase.dart';

import 'widget.dart';

String html({String lang, List<String> content}) => ElementWidget(
      'html',
      attributes: {'lang': lang},
      newlineAfterStart: true,
    )(content);

String head(List<String> content) => ElementWidget(
      'head',
      newlineAfterStart: true,
    )(content);

String link({LinkRel rel, String href, String type}) => ElementWidget(
      'link',
      attributes: {
        'rel': rel.name,
        'href': href,
        'type': type,
      },
      collapse: true,
    )();

String meta({String name, String content, String charset, String httpEquiv}) => ElementWidget(
      'meta',
      attributes: {
        'name': name,
        'content': content,
        'chacharset': content,
        'http-equiv': httpEquiv,
      },
      collapse: true,
    )();

String script({String type, String src, String content, bool defer = false}) => ElementWidget(
      'script',
      attributes: {
        'type': type,
        'defer': defer,
        'src': src,
      },
      newlineAfterStart: true,
    )(content);

String title(String content) => ElementWidget('title')(content);

String body(List<String> content) => ElementWidget(
      'body',
      newlineAfterStart: true,
    )(content);

String section(List<String> body, {String id, Map<String, String> style, List<String> classes}) => ElementWidget(
      'section',
      id: id,
      style: style,
      classes: classes,
      newlineAfterStart: true,
    )(body);

String article(List<String> body, {String id, Map<String, String> style, List<String> classes}) => ElementWidget(
      'article',
      id: id,
      style: style,
      classes: classes,
      newlineAfterStart: true,
    )(body);

String aside(List<String> body, {String id, Map<String, String> style, List<String> classes}) => ElementWidget(
      'aside',
      id: id,
      style: style,
      classes: classes,
      newlineAfterStart: true,
    )(body);

String div(
  List<String> body, {
  String id,
  Map<String, String> style,
  List<String> classes,
  Map<String, String> attributes,
}) =>
    ElementWidget(
      'div',
      id: id,
      style: style,
      classes: classes,
      newlineAfterStart: true,
      attributes: attributes,
    )(body);

String p(List<String> body, {String id, Map<String, String> style, List<String> classes}) => ElementWidget(
      'p',
      id: id,
      style: style,
      classes: classes,
      newlineAfterStart: true,
    )(body);

String label({
  @required String forId,
  String id,
  Map<String, String> style,
  List<String> classes,
  List<String> content,
}) =>
    ElementWidget(
      'label',
      attributes: {
        'for': forId,
      },
      id: id,
      style: style,
      classes: classes,
      newlineAfterStart: true,
    )(content);

String span(
  String body, {
  String id,
  Map<String, String> style,
  List<String> classes,
  newlineAfterEnd = false,
}) =>
    ElementWidget('span', id: id, style: style, classes: classes)(body);

String strong(
  String body, {
  String id,
  Map<String, String> style,
  List<String> classes,
  newlineAfterEnd = false,
}) =>
    ElementWidget('strong', id: id, style: style, classes: classes)(body);

String input({
  String id,
  Map<String, String> style,
  List<String> classes,
  String name,
  InputType type,
  String label,
  String value,
}) =>
    ElementWidget(
      'input',
      id: id,
      style: style,
      classes: classes,
      attributes: {
        'name': name,
        'type': type?.name,
        'label': label,
        'value': value,
      },
    )();

String a(String body, {String href, String id, Map<String, String> style, List<String> classes}) => ElementWidget(
      'a',
      id: id,
      style: style,
      classes: classes,
      attributes: {
        'href': href,
      },
      newlineAfterEnd: false,
    )(body);

String h1(String body, {String id, Map<String, String> style, List<String> classes}) =>
    ElementWidget('h1', id: id, style: style, classes: classes)(body);

String h2(String body, {String id, Map<String, String> style, List<String> classes}) =>
    ElementWidget('h2', id: id, style: style, classes: classes)(body);

String h3(String body, {String id, Map<String, String> style, List<String> classes}) =>
    ElementWidget('h3', id: id, style: style, classes: classes)(body);

String h4(String body, {String id, Map<String, String> style, List<String> classes}) =>
    ElementWidget('h4', id: id, style: style, classes: classes)(body);

String h5(String body, {String id, Map<String, String> style, List<String> classes}) =>
    ElementWidget('h5', id: id, style: style, classes: classes)(body);

String h6(String body, {String id, Map<String, String> style, List<String> classes}) =>
    ElementWidget('h6', id: id, style: style, classes: classes)(body);

String ul(List<String> body, {String id, Map<String, String> style, List<String> classes}) => ElementWidget(
      'ul',
      id: id,
      style: style,
      classes: classes,
      newlineAfterStart: true,
    )(body);

String ol(List<String> body, {String id, Map<String, String> style, List<String> classes}) => ElementWidget(
      'ol',
      id: id,
      style: style,
      classes: classes,
      newlineAfterStart: true,
    )(body);

String li(String body, {String id, Map<String, String> style, List<String> classes}) => ElementWidget(
      'li',
      id: id,
      style: style,
      classes: classes,
    )(body);

String br = ElementWidget('br', collapse: true)();

String canvas({String id, Map<String, String> style, List<String> classes, List<String> body}) => ElementWidget(
      'canvas',
      id: id,
      style: style,
      classes: classes,
    )(body);

enum LinkRel {
  alternate,
  author,
  dnsPrefetch,
  help,
  icon,
  license,
  next,
  pingback,
  preconnect,
  prefetch,
  preload,
  prerender,
  prev,
  search,
  stylesheet,
}

extension LinkRelName on LinkRel {
  String get name {
    var stringRepresentation = toString();
    return ReCase(stringRepresentation.substring(stringRepresentation.indexOf('.') + 1)).paramCase;
  }
}

enum InputType {
  email,
  number,
  password,
  search,
  tel,
  text,
  url,
  date,
  month,
}

extension InputTypeName on InputType {
  String get name => toString().substring(toString().indexOf('.') + 1);
}
