import 'package:templatr/widget.dart';

String tabGroup({
  String id,
  Map<String, String> style,
  List<String> classes,
  Placement placement,
  List<_Tab> tabs,
}) =>
    ElementWidget(
      'sl-tab-group',
      id: id,
      style: style,
      classes: classes,
      attributes: {'placement': placement?.name},
      template: (List<_Tab> tabs) =>
          '${tabs.map((tab) => _tab(tab.caption, panel: tab.name, disabled: tab.disabled)).join()}'
          '${tabs.map((tab) => _tabPanel(tab.content, name: tab.name)).join()}',
      newlineAfterStart: true,
    )(tabs);

_Tab tab(String caption, {String name, String content, bool disabled = false}) =>
    _Tab(caption, name, disabled, content);

String _tab(String caption, {String panel, bool disabled}) => ElementWidget(
      'sl-tab',
      attributes: {
        'panel': panel,
        'disabled': disabled,
      },
    )(caption);

String _tabPanel(String content, {String name}) => ElementWidget(
      'sl-tab-panel',
      attributes: {'name': name},
    )(content);

enum Placement {
  top,
  left,
  right,
  bottom,
}

class _Tab {
  final String caption;
  final String name;
  final bool disabled;
  final String content;

  _Tab(this.caption, this.name, this.disabled, this.content);
}

extension PlacementName on Placement {
  String get name => toString().substring(toString().indexOf('.') + 1);
}
