import 'package:templatr/html.dart';
import 'package:templatr/shoelace.dart' as sl;
import 'package:test/test.dart';

void main() {
  test('menu', () {
    expect(
        sl.menu(id: 'edit-menu', classes: [
          'sl-border'
        ], items: [
          sl.menuItem('Undo'),
          sl.menuItem('Redo', disabled: true),
          sl.menuItem('Keep history', value: 'keep-history', checked: true),
          sl.menuDivider,
          sl.menuLabel('Clipboard'),
          sl.menuItem('Cut'),
          sl.menuItem('Copy'),
          sl.menuItem('Paste', disabled: true),
        ]),
        '<sl-menu id="edit-menu" class="sl-border">\n'
        '<sl-menu-item>Undo</sl-menu-item>\n'
        '<sl-menu-item disabled>Redo</sl-menu-item>\n'
        '<sl-menu-item value="keep-history" checked>Keep history</sl-menu-item>\n'
        '<sl-menu-divider></sl-menu-divider>\n'
        '<sl-menu-label>Clipboard</sl-menu-label>\n'
        '<sl-menu-item>Cut</sl-menu-item>\n'
        '<sl-menu-item>Copy</sl-menu-item>\n'
        '<sl-menu-item disabled>Paste</sl-menu-item>\n'
        '</sl-menu>\n');
  });

  test('tab group', () {
    expect(
        sl.tabGroup(id: 'tabs', placement: sl.Placement.left, tabs: [
          sl.tab('Home', name: 'home', content: 'Welcome home'),
          sl.tab('Settings', name: 'settings', content: 'Set your settings'),
          sl.tab('Advanced', name: 'advanced', disabled: true, content: 'Advanced to the next level!'),
        ]),
        '<sl-tab-group id="tabs" placement="left">\n'
        '<sl-tab panel="home">Home</sl-tab>\n'
        '<sl-tab panel="settings">Settings</sl-tab>\n'
        '<sl-tab panel="advanced" disabled>Advanced</sl-tab>\n'
        '<sl-tab-panel name="home">Welcome home</sl-tab-panel>\n'
        '<sl-tab-panel name="settings">Set your settings</sl-tab-panel>\n'
        '<sl-tab-panel name="advanced">Advanced to the next level!</sl-tab-panel>\n'
        '</sl-tab-group>\n');
  });

  test('form', () {
    expect(
        sl.form(id: 'form', content: [
          sl.input(label: 'First name', name: 'firstName', type: InputType.text),
          br,
          sl.input(label: 'Last name', name: 'lastName', type: InputType.text),
          br,
          sl.select(label: 'Occupation', name: 'occupation', items: [
            sl.menuItem('Studying', value: 'studying'),
            sl.menuItem('Working', value: 'working'),
            sl.menuItem('Unemployed', value: 'unemployed'),
            sl.menuItem('Retired', value: 'retired'),
          ]),
          br,
          sl.checkbox('I accept the terms and conditions', name: 'policy', value: 'true'),
          br,
          sl.button('Register', type: sl.ButtonType.primary, submit: true),
        ]),
        '<sl-form id="form">\n'
        '<sl-input name="firstName" type="text" label="First name"></sl-input>\n'
            '<br>\n'
        '<sl-input name="lastName" type="text" label="Last name"></sl-input>\n'
            '<br>\n'
        '<sl-select name="occupation" label="Occupation">\n'
        '<sl-menu-item value="studying">Studying</sl-menu-item>\n'
        '<sl-menu-item value="working">Working</sl-menu-item>\n'
        '<sl-menu-item value="unemployed">Unemployed</sl-menu-item>\n'
        '<sl-menu-item value="retired">Retired</sl-menu-item>\n'
        '</sl-select>\n'
            '<br>\n'
        '<sl-checkbox name="policy" value="true">I accept the terms and conditions</sl-checkbox>\n'
            '<br>\n'
        '<sl-button type="primary" submit>Register</sl-button>\n'
        '</sl-form>\n');
  });

  test('dialog', () {
    expect(
        sl.dialog(label: 'Discard changes?', id: 'discard-changes', content: [
          'Are you sure you want to discard changes?',
          sl.button('Cancel', attributes: sl.footer),
          sl.button('Discard', attributes: sl.footer, type: sl.ButtonType.danger),
        ]),
        '<sl-dialog id="discard-changes" label="Discard changes?">\n'
        'Are you sure you want to discard changes?\n'
        '<sl-button slot="footer">Cancel</sl-button>\n'
        '<sl-button type="danger" slot="footer">Discard</sl-button>\n'
        '</sl-dialog>\n');
  });
}
